import 'package:flutter/material.dart';
import 'dart:ui' as ui;

class Utils {
  static String getEllipsizedText(
      String text, TextStyle textStyle, double maxWidth) {
    var tp = new TextPainter(
        text: TextSpan(text: text, style: textStyle),
        textDirection: ui.TextDirection.ltr);

    int i = 0;

    for (i = text.length; i > 0; i--) {
      tp.text = TextSpan(text: "${text.substring(0, i)}...", style: textStyle);

      tp.layout();

      if (tp.width <= maxWidth) break;
    }

    return "${text.substring(0, i)}${i == text.length ? "" : "..."}";
  }

  static String stringRepeat(String text, int count) {
    return List.filled(count, text).join();
  }

  static String leadingZeroInt(int value, int count) {
    String stringInt = value.toString();

    return stringInt.length > count
        ? stringInt
        : "${stringRepeat("0", count - stringInt.length)}$stringInt";
  }

  static parseStringToTextSpan(String fullString, List<String> findString) {
    String stringWithSmallestIndex =
        findWordWithSmallestIndex(fullString, findString, startFindFrom: 0);
    int index = fullString.indexOf(stringWithSmallestIndex);
    int startCropFrom = 0;
    List<TextSpan> textSpans = [];

    while (index != -1) {
      String croppedString = fullString.substring(startCropFrom, index);
      startCropFrom = index + stringWithSmallestIndex.length;

      textSpans.add(TextSpan(text: croppedString));
      textSpans.add(TextSpan(text: stringWithSmallestIndex));

      index++;

      if (index > fullString.length) break;

      stringWithSmallestIndex = findWordWithSmallestIndex(
          fullString, findString,
          startFindFrom: index);

      index = fullString.indexOf(stringWithSmallestIndex, index);
    }

    String croppedString =
        fullString.substring(startCropFrom, fullString.length);
    textSpans.add(TextSpan(text: croppedString));

    return textSpans;
  }

  static findWordWithSmallestIndex(String fullString, List<String> findString,
      {int startFindFrom}) {
    int tempSmallestIndex;
    int tempIndex;
    String wordWithSmallestIndex;
    int i = 0;

    while (i < findString.length) {
      tempIndex = fullString.indexOf(findString[i], startFindFrom);
      if (tempIndex == -1) {
        i++;
        continue;
      }
      wordWithSmallestIndex = tempSmallestIndex == null
          ? findString[i]
          : tempSmallestIndex < tempIndex
              ? wordWithSmallestIndex
              : findString[i];
      tempSmallestIndex = tempSmallestIndex == null
          ? tempIndex
          : tempSmallestIndex < tempIndex ? tempSmallestIndex : tempIndex;
      i++;
    }
    return wordWithSmallestIndex ?? findString[0];
  }
}
